import re

def parsePath(path):
  '''
  Regex parsing of typical JSONpath sytax. The searchJSON module is designed to work
  exclusively with this format.
  ex: $.Datavalue[?Cd="Losses"].Value 
      $.Locations[0].Value
  '''
  pathItems = re.split('\.|\[', path)[1:]
  return pathItems

def searchJSON(pathItems, root):
  '''
  Recursivly searches for and returns the terminating item 
  from a JSON object using a supplied list of parsed JSONpath 
  items. The returned item can be a dict, list, or value.
  '''
  debugPathItems = pathItems
  if len(pathItems) == 0:
    return root #once we've gone though all the JSONpath items
  if type(root) == dict:
    root = root.get(pathItems[0]) 
    return searchJSON(pathItems[1:], root) #pops off the first item in the list of JSONpath items after it's been used and recursively calls the module
  elif type(root) == list:
    if len(re.findall('[0-9]+\]', pathItems[0])) > 0: #checking whether or not we have been given an index integer. This happens when we know there is only one dict in the list
      return searchJSON(pathItems[1:], root[int(pathItems[0][:-1])])
    else:
      for object in root: #looping through dicts in the list looking for the filtering criteria described in the JSONpath item.
        filterPair = re.split('=', pathItems[0])
        if object.get(filterPair[0][1:]) == filterPair[1][1:-2]:
          root = object
          break #found what we're looking for so we can stop looping
      return searchJSON(pathItems[1:], root) #pops off the first item in the list of JSONpath items after it's been used and recursively calls the module
  else: 
    raise Exception('Oops, something went wrong in searchJSON     ', debugPathItems)

def getJSONrequestItem(root, path):
  '''
  Takes the object to search and the path to follow and returns 
  the terminating item as per the JSONpath.
  '''
  try:
    pathItems = parsePath(path)
    thing = searchJSON(pathItems, root)
  except IndexError:
    thing = None
  except:
    raise Exception('Unexpected error in getJSONrequestItem. The path was: ' + str(path))
  else:
    return thing

def updateJSONelement (path, value, root):
  '''
  Takes a path, value and object in which we want to update a value
  and finds an item described by the path and updates its value
  with the supplied value.
  '''
  try:
    pathItems = parsePath(path)
    containingDict = searchJSON(pathItems[:-1], root)
    if type(value) != list and type(value) != dict:
      containingDict[pathItems[-1]] = str(value)
    else:
      containingDict[pathItems[-1]] = value
  except:
    raise Exception("Dang, update failed for path: " + str(path))

def writeNewJSONelement (newElementName, value, path, root):
  '''
  Takes a new element name, value, path, and object in which we
  want to insert an item. A dict, list, or value can be inserted
  based on the location in the object we are trying to do the insert.
  In order to add a new list (array) value must be set to "newArray".
  '''
  pathString = path
  pathItems = parsePath(path)
  root = searchJSON(pathItems, root)
  if type(value) != list and type(value) != dict:
    value = str(value)
  if value == 'newArray':
    root[newElementName] = []
  elif pathString[-1] != ']' and len(pathString) > 1:
    root.append({'Cd' : newElementName, 'Value' : value})
  else:
    root[newElementName] = value


 